﻿using LiteDB;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BaiduYunSync
{
    class LiteDbHelper
    {

        private static string dbUrl = "db.db";

        /// <summary>
        /// 本地保存更新信息，防止频繁读写文件
        /// </summary>
        private static List<UpdateInfoDao> updateInfoList;

        /// <summary>
        /// 同步文件锁
        /// </summary>
        private static object updateInfoLocker = new object();

        static LiteDbHelper()
        {
            Init();
        }

        /// <summary>
        /// 初始化数据库相关内容
        /// </summary>
        public static void Init()
        {
            if (updateInfoList == null)
            {
                using (var db = new LiteDatabase(dbUrl))
                {
                    var col = db.GetCollection<UpdateInfoDao>("updateinfo");
                    var list = col.FindAll();
                    updateInfoList = (list == null ?  new List<UpdateInfoDao>() : list.ToList());
                }
            }
        }

        private static UserInfoDao userInfo;

        /// <summary>
        /// 获取或设置用户信息
        /// </summary>
        public static UserInfoDao UserInfo
        {
            get
            {
                if (userInfo == null)
                {
                    using (var db = new LiteDatabase(dbUrl))
                    {
                        var col = db.GetCollection<UserInfoDao>("userinfo");
                        userInfo = col.FindOne(Query.All());
                        if (userInfo == null)
                        {
                            userInfo = new UserInfoDao();
                        }
                    }
                }
                return userInfo;
            }
            set
            {
                using (var db = new LiteDatabase(dbUrl))
                {
                    var col = db.GetCollection<UserInfoDao>("userinfo");
                    if (value.Id == 0)
                    {
                        col.Insert(value);
                    }
                    else
                    {
                        col.Update(value);
                    }
                    userInfo = value;
                }
            }
        }

        /// <summary>
        /// 插入或更新需要同步的文件夹
        /// </summary>
        /// <param name="dao"></param>
        public static void UpdateSyncItem(SyncItemsDao dao)
        {
            using (var db = new LiteDatabase(dbUrl))
            {
                var col = db.GetCollection<SyncItemsDao>("syncitem");
                if (dao.Id == 0)
                {
                    col.Insert(dao);
                }
                else
                {
                    col.Update(dao);
                }
            }
        }

        /// <summary>
        /// 获取需要同步的文件夹列表
        /// </summary>
        /// <returns>同步文件夹信息</returns>
        public static List<SyncItemsDao> GetAllSyncItems()
        {
            using (var db = new LiteDatabase(dbUrl))
            {
                var col = db.GetCollection<SyncItemsDao>("syncitem");
                return col.FindAll().ToList();
            }
        }

        /// <summary>
        /// 删除同步文件夹
        /// </summary>
        /// <param name="dao"></param>
        public static void DeleteSyncItem(SyncItemsDao dao)
        {
            using (var db = new LiteDatabase(dbUrl))
            {
                var col = db.GetCollection<SyncItemsDao>("syncitem");
                col.Delete(dao.Id);
            }
        }

        /// <summary>
        /// 获取所有的更新信息
        /// </summary>
        /// <returns></returns>
        public static UpdateInfoDao[] GetAllUpdateInfos()
        {
            lock (updateInfoLocker)
            {
                return updateInfoList.ToArray();
            }
        }

        /// <summary>
        /// 插入新的更新信息
        /// </summary>
        /// <param name="dao"></param>
        public static void InsertUpdateInfo(UpdateInfoDao dao)
        {
            using (var db = new LiteDatabase(dbUrl))
            {
                var col = db.GetCollection<UpdateInfoDao>("updateinfo");
                col.Insert(dao);
            }
            lock (updateInfoLocker)
                updateInfoList.Add(dao);
        }

        /// <summary>
        /// 删除更新信息
        /// </summary>
        /// <param name="dao"></param>
        public static void DeleteUpdateInfo(UpdateInfoDao dao)
        {
            using (var db = new LiteDatabase(dbUrl))
            {
                var col = db.GetCollection<UpdateInfoDao>("updateinfo");
                col.Delete(dao.Id);
            }
            lock (updateInfoLocker)
                updateInfoList.Remove(dao);
        }

        /// <summary>
        /// 获取需更新数量
        /// </summary>
        /// <returns>数量</returns>
        public static int GetUpdateInfoCount()
        {
            lock (updateInfoLocker)
                return updateInfoList.Count();
        }

        /// <summary>
        /// 判断文件是否已存在
        /// </summary>
        /// <param name="path">文件全路径</param>
        /// <returns></returns>
        public static bool ContainsPath(string path)
        {
            lock (updateInfoLocker)
                return updateInfoList.Select(x => x.Path).Contains(path);
        }

        /// <summary>
        /// 根据路径获取对应的更新信息
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static UpdateInfoDao GetUpdateInfoFormPath(string path)
        {
            if (!ContainsPath(path))
            {
                return null;
            }
            lock (updateInfoLocker)
                return updateInfoList.Where(x => x.Path == path).First();
        }
    }
}
